#include "DLList.h"
#include <iostream>

DLList::DLList() {
	head = new Node();
	tail = head;
	current = head;
	(*head).next = 0;
	(*head).previous = 0;
	length = 0;
}

// Accessors

void DLList::print() {
	Node* i = head;
	if (length == 0) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else {
		while ((*i).next != NULL) {
			std::cout << "Data: " << (*i).data << std::endl;
			i = (*i).next;
		}
	}
}

void DLList::printHead() {
	if ((*head).next == NULL) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else {
		std::cout << "Head: " << (*head).data << std::endl;
	}
}

void DLList::printCurrent() {
	if ((*current).next == NULL && (*current).previous == NULL) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else {
		std::cout << "Current: " << (*current).data << std::endl;
	}
}

void DLList::printTail() {
	if (length == 0) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else {
		std::cout << "Tail: " << (*tail).data << std::endl;
	}
}

void DLList::traverseForward() {
	if (length == 0) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else if (length == 1) {
		std::cout << "ONLY ONE NODE!" << std::endl;
	} else if (current == tail) {
		std::cout << "ERROR: GOT END OF LIST!" << std::endl;
	} else {
		std::cout << "Traversing forward..." << std::endl;
		current = (*current).next;
		std::cout << "Current: " << (*current).data << std::endl;
	}
}

void DLList::traverseBackward() {
	if (length == 0) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else if (length == 1) {
		std::cout << "ONLY ONE NODE!" << std::endl;
	} else if (current == head) {
		std::cout << "ERROR: GOT BEGINNING OF LIST!" << std::endl;
	} else {
		std::cout << "Traversing backward..." << std::endl;
		current = (*current).previous;
		std::cout << "Current: " << (*current).data << std::endl;
	}
}

int DLList::getLength() {
		return length;
}

// Modifiers

void DLList::pushFront(int newData) {
	Node* i;
	if (length == 0) {
		(*head).data = newData;
		(*head).previous = new Node();
		(*head).next = new Node();
		tail = head;
	} else if (length == 1) {
		head = (*head).previous;
		(*head).data = newData;
		(*head).previous = new Node();
		(*head).next = tail;
	} else {
		i = (*head).previous;
		(*i).data = newData;
		(*i).next = head;
		(*i).previous = new Node();
		head = i;
	}
	length++;
}

void DLList::pushBack(int newData) {
	Node* i;
	if (length == 0) {
		(*head).data = newData;
		(*head).previous = new Node();
		(*head).next = new Node();
		tail = head;
	} else if (length == 1) {
		tail = new Node();
		(*tail).data = newData;
		(*tail).previous = head;
		(*head).next = tail;
		(*tail).previous = head;
		(*tail).next = new Node();
	} else {
		i = (*tail).next;
		(*i).data = newData;
		(*i).previous = tail;
		(*i).next = new Node();
		tail = i;
	}
	length++;
}

void DLList::popFront() {
	if (length == 0) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else if (length == 1) {
		std::cout << "ONLY ONE NODE!" << std::endl;
	} else {
		if (current == head) {
			current = (*head).next;
		}
		head = (*head).next;
		(*head).next->previous = head;
		std::cout << "Deleting: " << (*head).previous->data << std::endl;
		delete [] (*head).previous;
		(*head).previous = new Node();
		length--;
	}
}

void DLList::popBack() {
	if (length == 0) {
		std::cout << "EMPTY LIST!" << std::endl;
	} else if (length == 1) {
		std::cout << "ONLY ONE NODE!" << std::endl;
	} else {
		if (current == tail) {
			current = (*tail).previous;
		}
		tail = (*tail).previous;
		(*tail).previous->next = tail;
		std::cout << "Deleting: " << (*tail).next->data << std::endl;
		delete [] (*tail).next;
		(*tail).next = new Node();
		length--;
	}
}

void printOptions() {
	std::cout << "1) Add node to front (Push front)" << std::endl;
	std::cout << "2) Add node to back (Push back)" << std::endl;
	std::cout << "3) Delete node from front (Pop front)" << std::endl;
	std::cout << "4) Delete node from back (Pop back)" << std::endl;
	std::cout << "5) Print head node" << std::endl;
	std::cout << "6) Print current node" << std::endl;
	std::cout << "7) Print tail node" << std::endl;
	std::cout << "8) Print all nodes" << std::endl;
	std::cout << "9) Traverse forward" << std::endl;
	std::cout << "10) Traverse backward" << std::endl;
	std::cout << "11) Get list length" << std::endl;
	std::cout << "12) Print options" << std::endl;
	std::cout << "13) Quit" << std::endl;
}

int main() {
	DLList dl;
	bool lock = true;
	int buffer;

	std::cout << "Doubly Linked List Tester 1.0" << std::endl;
	printOptions();
	while (lock) {
		std::cout << "Choose an option: ";
		std::cin >> buffer;
		switch (buffer) {
			case 1:
				std::cout << "Enter node value: ";
				std::cin >> buffer;
				dl.pushFront(buffer);
				break;
			case 2:
				std::cout << "Enter node value: ";
				std::cin >> buffer;
				dl.pushBack(buffer);
				break;
			case 3: dl.popFront(); break;
			case 4: dl.popBack(); break;
			case 5: dl.printHead(); break;
			case 6: dl.printCurrent(); break;
			case 7: dl.printTail(); break;
			case 8: dl.print(); break;
			case 9: dl.traverseForward(); break;
			case 10: dl.traverseBackward(); break;
			case 11: 
				std::cout << "Length: " << dl.getLength() << std::endl; break;
			case 12: printOptions(); break;
			case 13: lock = false; break;
			default: break;
		}
	}

	std::cout << "Goodbye!" << std::endl;
	return 0;
}
